"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyFilter = void 0;
const uniquifyArray_1 = require("../general/uniquifyArray");
const toArray_1 = require("../general/toArray");
const namespaces_1 = require("../../data/namespaces");
const schemaModules_1 = require("../../data/schemaModules");
const checkFilterValidity_1 = require("./checkFilterValidity");
function applyFilter(paramObj) {
    const { data, filter, graph } = paramObj;
    let usedFilter = filter;
    if (!usedFilter && graph.defaultFilter) {
        usedFilter = graph.defaultFilter;
    }
    if (!Array.isArray(data) ||
        data.length === 0 ||
        !usedFilter ||
        Object.keys(usedFilter).length === 0) {
        return (0, uniquifyArray_1.uniquifyArray)(data);
    }
    (0, checkFilterValidity_1.checkFilterValidity)(usedFilter);
    let terms = createTermInstances((0, uniquifyArray_1.uniquifyArray)(data), graph);
    terms = filterTermType(terms, usedFilter.termType, false);
    terms = filterTermType(terms, usedFilter.termTypeExclude, true);
    terms = filterFromVocabulary(terms, usedFilter.fromVocabulary, graph, false);
    terms = filterFromVocabulary(terms, usedFilter.fromVocabularyExclude, graph, true);
    terms = filterSuperseded(terms, usedFilter);
    terms = filterSchemaModule(terms, usedFilter.schemaModule, false);
    terms = filterSchemaModule(terms, usedFilter.schemaModuleExclude, true);
    return terms.map(t => t.getIRI("Compact"));
}
exports.applyFilter = applyFilter;
function createTermInstances(data, graph) {
    const result = [];
    for (const termIri of data) {
        try {
            result.push(graph.getTerm(termIri, {}));
        }
        catch (e) {
            console.log(e);
        }
    }
    return result;
}
function filterFromVocabulary(terms, fromVocabularyFilter, graph, excludeCheck) {
    if (!fromVocabularyFilter) {
        return terms;
    }
    const context = graph.context;
    const namespaces = (0, toArray_1.toArray)(fromVocabularyFilter);
    for (let v = 0; v < namespaces.length; v++) {
        for (let vi = 0; vi < Object.keys(context).length; vi++) {
            if (context[Object.keys(context)[vi]] === namespaces[v]) {
                namespaces[v] = Object.keys(context)[vi];
                break;
            }
        }
    }
    const result = [];
    for (const term of terms) {
        const termIri = term.getIRI("Compact");
        const matchFound = namespaces.some(ns => termIri.startsWith(ns + ":"));
        if (matchFound && !excludeCheck) {
            result.push(term);
        }
        else if (!matchFound && excludeCheck) {
            result.push(term);
        }
    }
    return result;
}
function filterSuperseded(terms, filter) {
    if (filter.isSuperseded === undefined) {
        return terms;
    }
    const result = [];
    for (const term of terms) {
        if (!filter.isSuperseded && term.isSupersededBy() === null) {
            result.push(term);
        }
        else if (filter.isSuperseded && term.isSupersededBy() !== null) {
            result.push(term);
        }
    }
    return result;
}
function filterTermType(terms, termTypeFilter, excludeCheck) {
    if (termTypeFilter === undefined) {
        return terms;
    }
    const termTypeArray = (0, toArray_1.toArray)(termTypeFilter);
    const invalidTermType = termTypeArray.find((el) => !(0, namespaces_1.isTermTypeLabelValue)(el));
    if (invalidTermType) {
        throw new Error("Invalid filter.termType " + invalidTermType);
    }
    const result = [];
    for (const term of terms) {
        if ((termTypeArray.includes(term.getTermTypeLabel()) && !excludeCheck) ||
            (!termTypeArray.includes(term.getTermTypeLabel()) && excludeCheck)) {
            result.push(term);
        }
    }
    return result;
}
function filterSchemaModule(terms, schemaModuleFilter, excludeCheck) {
    if (schemaModuleFilter === undefined) {
        return terms;
    }
    const schemaModuleArray = (0, toArray_1.toArray)(schemaModuleFilter);
    const invalidSchemaModule = schemaModuleArray.find((el) => !(0, schemaModules_1.isSchemaModule)(el));
    if (invalidSchemaModule) {
        throw new Error("Invalid filter.schemaModule " + invalidSchemaModule);
    }
    const result = [];
    for (const term of terms) {
        const schemaModuleMatch = (0, schemaModules_1.getSchemaModuleMatch)(term.getVocabulary());
        if (!schemaModuleMatch ||
            (schemaModuleArray.includes(schemaModuleMatch) && !excludeCheck) ||
            (!schemaModuleArray.includes(schemaModuleMatch) && excludeCheck)) {
            result.push(term);
        }
    }
    return result;
}
//# sourceMappingURL=applyFilter.js.map